using OpenTK.Graphics.OpenGL4;
using OpenTK.Windowing.Common;
using OpenTK.WinForms;
using System.Diagnostics;
using System.Runtime;

namespace OpenTkDemo
{
    public partial class Form1 : Form
    {
        public GLControl glControl;

        // Timers
        public Thread backgroundThread;
        public System.Windows.Forms.Timer uiTimer = new();
        public bool uiTimerTicking;

        // Frame rate measurement.
        public Stopwatch measureWatch = new();
        public const int SET_FRAME_RATE_MS = 10;
        public List<double> frameAveList = new();
        public double achievedFrameRateMs;

        // Use ui timer or background timer for drawing.
        public const bool DRAW_WITH_UI_THREAD = false;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Shown(object sender, EventArgs e)
        {
            // Initialize control and add to existing panel
            // 32 bit depth buffer requires rendering to non-default frame buffer (future option).
            // 8 stencil buffer required for proper stenciling if used.
            // 8 antialiasing level better than 4 for certain edges.
            glControl = new(new GLControlSettings() { NumberOfSamples = 8, API = ContextAPI.OpenGL, APIVersion = new Version("3.3.0.0"), Profile = ContextProfile.Core });
            glControl.Dock = DockStyle.Fill;
            tabPage1.Controls.Add(glControl);

            // Init foreground timer.
            uiTimer.Interval = SET_FRAME_RATE_MS;
            uiTimer.Tick += UiTimerTick;
            uiTimer.Start();

            // Unbind from the UI thread.
            if (!DRAW_WITH_UI_THREAD)
            {
                // Begin context switch.
                glControl.Context.MakeNoneCurrent();

                // Init background timer.
                backgroundThread = new(new ThreadStart(ThreadTick));
                backgroundThread.IsBackground = true;
                backgroundThread.Start();
            }
        }

        // Draw with UI Timer.
        private void UiTimerTick(object sender, EventArgs e)
        {
            // Prevent multiple ticks.
            if (!uiTimerTicking)
            {
                uiTimerTicking = true;

                // Draw with UI timer if set.
                if (DRAW_WITH_UI_THREAD)
                    Draw();

                // Show frame rate.
                lblFrameRateMs.Text = achievedFrameRateMs.ToString("F3");

                // Allow another tick.
                uiTimerTicking = false;
            }
        }

        // Draw with background thread.
        private void ThreadTick()
        {
            while (true)
            {
                Draw();
                Thread.Sleep(SET_FRAME_RATE_MS);
            }
        }

        // Draw simple.
        private void Draw()
        {
            try
            {
                // Measure frame rate.
                measureWatch.Restart();

                // Makes the graphics context associated with the glwindow current on the calling thread.
                // This allows drawing to occur on a non-ui thread.
                if (!DRAW_WITH_UI_THREAD && !glControl.Context.IsCurrent)
                    glControl.MakeCurrent();

                GL.ClearColor(Color.Red);
                GL.Clear(ClearBufferMask.ColorBufferBit | ClearBufferMask.DepthBufferBit);

                //  The SwapBuffers() method "swaps" the front and back buffers 
                //  and displays the result of the rendering commands to the user. 
                //  The contents of the new back buffer are undefined after the call to SwapBuffers().
                glControl.SwapBuffers();

                // Store achieved frame rate average.
                measureWatch.Stop();
                frameAveList.Add(measureWatch.Elapsed.TotalMilliseconds);
                if (frameAveList.Count > 20)
                    frameAveList.RemoveAt(0);
                achievedFrameRateMs = frameAveList.Average();
            }
            catch (Exception ex)
            {
            }
        }
    }
}